from serial import *
from serial.tools import list_ports
from tkinter import ttk
from tkinter.messagebox import *
import tkinter as tk
import threading
import time

baudrate = 9600


# ----------------------------------------------- #
ser: Serial = None

def loop() -> None:
    while(True):
        # Update port list
        port_chose['values'] = [x.device for x in list_ports.comports()]
        
        if(ser is None or not ser.isOpen()):
            continue

        try:
            data = ser.read(1)
            ser.reset_input_buffer()
            print(data)
            text_a.set(f'{int.from_bytes(data)}')
            entry_a.update()
            time.sleep(0.1)
        except UnicodeDecodeError:
            # print("UnicodeDecodeError")
            pass
            
        ans['text'] = str(int(text_a.get()) + int(text_b.get()))
    

def connect():
    global ser
    if ser is not None:
        ser.close()
        del ser
    
    if com_port.get() == "":
        showerror("Connecting Error", "Please select a port to connect")
        return
    ser = Serial(com_port.get(), baudrate, timeout=0, writeTimeout=0)

def send() -> None:
    if ser is None or not ser.isOpen():
        showerror("Sending Error", "Please connect to a port first")
        return
    ser.write(int(entry_b.get()).to_bytes())

# --------------------- GUI --------------------- #
root = tk.Tk()
root.title("UART")
root.geometry('360x300')
root.minsize(360, 300)

ctrl_frame = tk.Frame(root)
ctrl_frame.pack(side=tk.TOP, fill=tk.X)

com_port = tk.StringVar()
port_chose = ttk.Combobox(ctrl_frame, textvariable=com_port) # 下拉式清單
port_chose.pack(side=tk.LEFT, padx=5, pady=3)

connect_btn = tk.Button(ctrl_frame, text="Connect", command=connect)
connect_btn.pack(side=tk.LEFT)

frame1 = tk.Frame(root)
frame1.pack(side=tk.TOP, fill=tk.BOTH, padx=30, pady=30)

text_a = tk.StringVar()
entry_a = tk.Entry(frame1, width=4, font=('Arial', 30), state=tk.DISABLED, 
                   textvariable=text_a, justify=tk.RIGHT)
entry_a.pack(side=tk.LEFT, padx=10)

tk.Label(frame1, text=" + ", font=('Arial', 40)).pack(side=tk.LEFT)

text_b = tk.StringVar()
entry_b = tk.Spinbox(frame1, width=4, font=('Arial', 30), textvariable=text_b, 
                     justify=tk.RIGHT, from_=0, to=255, command=send)
entry_b.pack(side=tk.LEFT, padx=10)

frame2 = tk.Frame(root)
frame2.pack(side=tk.TOP, fill=tk.BOTH, padx=30, pady=30)

tk.Label(frame2, text=" = ", font=('Arial', 40)).pack(side=tk.LEFT)
ans = tk.Label(frame2, text="0", font=('Arial', 40))
ans.pack(side=tk.LEFT)


# --------------------- Main --------------------- #
t = threading.Thread(target=loop)
t.daemon = True
t.start()

root.mainloop()



